@@ -13,7 +13,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * $Id: cert_info.c 407 2010-04-07 13:36:19Z ludovic.rousseau $
+ * $Id: cert_info.c 481 2011-01-22 17:40:34Z ludovic.rousseau $
  */
 
 #ifndef __CERT_INFO_C_
@@ -628,7 +628,7 @@ static int int_append(unsigned char *pt, int n) {
 	*pt++= (n&0xff000000) >>24;
 	*pt++= (n&0x00ff0000) >>16;
 	*pt++= (n&0x0000ff00) >>8;
-	*pt++= (n&0x000000ff) >>0;
+	*pt  = (n&0x000000ff) >>0;
 	return 4;
 }
 
@@ -646,12 +646,12 @@ static int BN_append(unsigned char *pt, BIGNUM *bn) {
 	int size= 1 + BN_num_bytes(bn);
 	unsigned char *buff;
 	if(BN_is_zero(bn)) {
-		res= int_append(pt,0);pt+=res;
+		res= int_append(pt,0);
 		return res;
 	}
 	buff=malloc(size);
 	*buff=0x00;
-	res=BN_bn2bin(bn,buff+1);
+	BN_bn2bin(bn,buff+1);
 	/* TODO: handle error condition */
 	extrabyte=( buff[1] & 0x80 )? 0:1;
 	res= int_append(pt,size-extrabyte); pt+=res;
