@@ -12,7 +12,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * $Id: pkcs11_lib.c 453 2010-10-15 07:34:12Z ludovic.rousseau $
+ * $Id: pkcs11_lib.c 478 2011-01-22 17:33:07Z ludovic.rousseau $
  */
 
 #define __PKCS11_LIB_C__
@@ -1320,7 +1320,6 @@ int wait_for_token_by_slotlabel(pkcs11_handle_t *h,
 {
   int rv;
 
-  rv = -1;
   do {
     /* see if the card we're looking for is inserted */
     rv = find_slot_by_slotlabel_and_tokenlabel (h, wanted_slot_label,
@@ -1343,7 +1342,6 @@ int wait_for_token(pkcs11_handle_t *h,
 {
   int rv;
 
-  rv = -1;
   do {
     /* see if the card we're looking for is inserted */
     rv = find_slot_by_number_and_label (h, wanted_slot_id,  wanted_token_label,
@@ -1444,13 +1442,14 @@ int close_pkcs11_session(pkcs11_handle_t *h)
   /* close user-session */
   DBG("logout user");
   rv = h->fl->C_Logout(h->session);
-  if (rv != CKR_OK && rv != CKR_USER_NOT_LOGGED_IN) {
+  if (rv != CKR_OK && rv != CKR_USER_NOT_LOGGED_IN
+	  && rv != CKR_FUNCTION_NOT_SUPPORTED) {
     set_error("C_Logout() failed: 0x%08lX", rv);
     return -1;
   }
   DBG("closing the PKCS #11 session");
   rv = h->fl->C_CloseSession(h->session);
-  if (rv != CKR_OK) {
+  if (rv != CKR_OK && rv != CKR_FUNCTION_NOT_SUPPORTED) {
     set_error("C_CloseSession() failed: 0x%08lX", rv);
     return -1;
   }
