@@ -13,7 +13,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * $Id: pklogin_finder.c 358 2008-11-06 14:28:46Z ludovic.rousseau $
+ * $Id: pklogin_finder.c 445 2010-08-14 16:19:36Z ludovic.rousseau $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -49,7 +49,7 @@ int main(int argc, const char **argv) {
     }
 
   /* call configure routines */
-  configuration = pk_configure(argc,argv);
+  configuration = pk_configure(argc - 1, argv + 1);
   if (!configuration ) {
 	DBG("Error setting configuration parameters");
 	return 1;
@@ -145,8 +145,8 @@ int main(int argc, const char **argv) {
       DBG("Trying to deduce login from certificate");
       user=find_user(x509);
       if (!user) {
-          DBG1("find_user() failed: %s", get_error());
-	  break;
+          DBG2("find_user() failed for certificate #%d: %s", i + 1, get_error());
+	  continue;	/* with next certificate */
       } else {
           DBG1("Certificate is valid and maps to user %s",user);
 	  printf("%s\n",user);
